package com.sendFile.protocol.SSH;

import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;
import java.io.File;
import java.util.Properties;
import java.util.Vector;
import com.moveFile.Movefile;

public class SSHProtocol {
  String user;
  String password;
  String host;
  int port;
  String local_fileName;
  String remote_filePath;
  String remote_fileName;
  String archive_path;
  String local_folder_path_bkp;

  public SSHProtocol(
      String user,
      String password,
      String host,
      int port,
      String local_fileName,
      String remote_filePath,
      String remote_fileName,
      String archive_path) {
    this.user = user;
    this.password = password;
    this.host = host;
    this.port = port;
    this.local_fileName = local_fileName;
    this.remote_filePath = remote_filePath;
    this.remote_fileName = remote_fileName;
    this.archive_path = archive_path;
  }

  public void sendFileSSH() {
    try {
      System.out.println("Going to SEND files from remote Server via SSH");
      JSch jsch = new JSch();
      System.out.println(
          "Creating SSH Session: user@host:port |--->"
              + this.user
              + "@"
              + this.host
              + ":"
              + this.port);
      Session session = jsch.getSession(this.user, this.host, this.port);
      session.setPassword(this.password);

      Properties config = new Properties();
      config.put("StrictHostKeyChecking", "no");
      session.setConfig(config);
      System.out.println("Establishing Connection...");
      session.connect();
      System.out.println("Connection established.");
      System.out.println("Creating SFTP Channel.");
      ChannelSftp sftpChannel = (ChannelSftp) session.openChannel("sftp");
      sftpChannel.connect();
      System.out.println("SFTP Channel created.");
      System.out.println(
          "SFTP -- Local File Name: "
              + this.local_fileName
              + " Remote File Name: "
              + this.remote_fileName);

      String[] fileTokens = this.remote_fileName.split("-");

      System.out.println(
          "Going to create brancheise folder:" + fileTokens[0] + " at " + this.remote_filePath);
      sftpChannel.cd(this.remote_filePath);
      try {
        sftpChannel.mkdir(fileTokens[0]);
      } catch (Exception ex) {
        System.out.println("Folder not created: " + ex.getMessage());
      }
      sftpChannel.cd(fileTokens[0]);

      sftpChannel.put(this.local_fileName, fileTokens[1]);
      System.out.println("File copied to remote server Successfully, archiving file");

      Movefile mv = new Movefile();
      mv.movefile(new File(this.local_fileName), this.archive_path);
      System.out.println("File archived to: " + this.archive_path);
      sftpChannel.disconnect();
      session.disconnect();
      System.out.println("Disconnect SFTP Channel");
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public void fetchFileSSH(String remote_report_pattern) {
    try {
      String local_folder_path_bkp = "/home/ec2-user/PRODFILES/reportFiles/processed_bkp";
      System.out.println("Going to FETCH files from remote Server");
      JSch jsch = new JSch();
      System.out.println(
          "Creating SSH Session: user@host:port |--->"
              + this.user
              + "@"
              + this.host
              + ":"
              + this.port);
      Session session = jsch.getSession(this.user, this.host, this.port);
      session.setPassword(this.password);

      Properties config = new Properties();
      config.put("StrictHostKeyChecking", "no");
      session.setConfig(config);
      System.out.println("Establishing Connection...");
      session.connect();
      System.out.println("Connection established.");
      System.out.println("Creating SFTP Channel.");
      ChannelSftp sftpChannel = (ChannelSftp) session.openChannel("sftp");
      sftpChannel.connect();
      System.out.println("SFTP Channel created.");

      sftpChannel.cd(this.remote_filePath);
      System.out.println(sftpChannel.pwd());
      String rootPath = sftpChannel.pwd();
      Vector<ChannelSftp.LsEntry> list = sftpChannel.ls(sftpChannel.pwd());

      for (ChannelSftp.LsEntry entry : list) {

        System.out.println(
            "Current File/Folder: " + entry.getFilename() + " isDir" + entry.getAttrs().isDir());
        if (entry.getAttrs().isDir()) {
          if (entry.getFilename().equals(".") || entry.getFilename().equals("..")) {
            System.out.println("Skipping . and ..");
            continue;
          }
          System.out.println("Found folder: " + entry.getFilename());

          System.out.println("cd: " + rootPath + "/" + entry.getFilename());
          sftpChannel.cd(rootPath + "/" + entry.getFilename());
          System.out.println(sftpChannel.pwd());
          Vector<ChannelSftp.LsEntry> subDirFileList = sftpChannel.ls(remote_report_pattern);
          for (ChannelSftp.LsEntry subDirFiles : subDirFileList) {
            if (!(new File(local_folder_path_bkp + "/" + subDirFiles.getFilename())).exists()) {
              sftpChannel.get(
                  subDirFiles.getFilename(), this.local_fileName + subDirFiles.getFilename());
              System.out.println(
                  "File: " + subDirFiles.getFilename() + " copied to :" + this.local_fileName);

              continue;
            }
            System.out.println(
                "File: " + subDirFiles.getFilename() + " already present in local path");
          }
        }
      }

      sftpChannel.disconnect();
      session.disconnect();
      System.out.println("Disconnect SFTP Channel");
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}